# -*- coding: utf-8 -*-
from plone.app.form.widgets import ListSequenceWidget
from plone.app.form.widgets import SequenceWidget
from plone.app.form.widgets import TupleSequenceWidget
from zope.formlib.interfaces import MissingInputError
from zope.formlib.interfaces import WidgetInputError
from zope.formlib.tests.test_sequencewidget import SequenceWidgetTest as BaseSequenceWidgetTest
from zope.formlib.widget import CustomWidgetFactory
from zope.formlib.widgets import ObjectWidget
from zope.interface import implements
from zope.interface import Interface
from zope.publisher.browser import TestRequest
from zope.schema import List
from zope.schema import TextLine
from zope.schema import Tuple


class SequenceWidgetTest(BaseSequenceWidgetTest):
    """Documents and tests the tuple and list (sequence) widgets.

        >>> from zope.interface.verify import verifyClass
        >>> verifyClass(IInputWidget, TupleSequenceWidget)
        True
        >>> verifyClass(IInputWidget, ListSequenceWidget)
        True
    """

    _WidgetFactory = TupleSequenceWidget

    def setUpContent(self, desc=u'', title=u'Foo Title'):

        class ITestContent(Interface):
            foo = self._FieldFactory(
                title=title,
                description=desc,
            )

        class TestObject(object):
            implements(ITestContent)
            foo = ('one', 'two')

        self.content = TestObject()
        self.field = ITestContent['foo'].bind(self.content)
        self.request = TestRequest(HTTP_ACCEPT_LANGUAGE='pl')
        self.request.form['field.foo'] = u'Foo Value'
        self._widget = self._WidgetFactory(
            self.field,
            self.field.value_type,
            self.request
        )

    def test_customWidgetFactory(self):
        """Verify that the widget can be constructed via the CustomWidgetFactory
        (Issue #293)
        """

        value_type = TextLine(__name__=u'bar')
        self.field = List(__name__=u'foo', value_type=value_type)
        request = TestRequest()

        # set up the custom widget factory and verify that it works
        sw = CustomWidgetFactory(ListSequenceWidget)
        widget = sw(self.field, request)
        assert widget.subwidget is None
        assert widget.context.value_type is value_type

        # set up a variant that specifies the subwidget to use and verify it
        class PollOption(object):
            pass

        ow = CustomWidgetFactory(ObjectWidget, PollOption)
        sw = CustomWidgetFactory(ListSequenceWidget, subwidget=ow)
        widget = sw(self.field, request)
        assert widget.subwidget is ow
        assert widget.context.value_type is value_type

    def test_subwidget(self):
        """This test verifies that the specified subwidget is not ignored.
        (Issue #293)
        """
        self.field = List(__name__=u'foo',
                          value_type=TextLine(__name__=u'bar'))
        request = TestRequest()

        class PollOption(object):
            pass

        ow = CustomWidgetFactory(ObjectWidget, PollOption)
        widget = SequenceWidget(
            self.field, self.field.value_type, request, subwidget=ow)
        assert widget.subwidget is ow

    def test_list(self):
        self.field = List(
            __name__=u'foo',
            value_type=TextLine(__name__=u'bar'))
        request = TestRequest()
        widget = ListSequenceWidget(
            self.field, self.field.value_type, request)
        self.assertFalse(widget.hasInput())
        self.assertRaises(MissingInputError, widget.getInputValue)

        request = TestRequest(form={'field.foo.add': u'Add bar',
                                    'field.foo.count': u'0'})
        widget = ListSequenceWidget(
            self.field, self.field.value_type, request)
        self.assert_(widget.hasInput())
        self.assertRaises(WidgetInputError, widget.getInputValue)

        request = TestRequest(form={'field.foo.0.bar': u'Hello world!',
                                    'field.foo.count': u'1'})
        widget = ListSequenceWidget(
            self.field, self.field.value_type, request)
        self.assert_(widget.hasInput())
        self.assertEqual(widget.getInputValue(), [u'Hello world!'])

    def test_new(self):
        request = TestRequest()
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        self.assertFalse(widget.hasInput())
        self.assertRaises(MissingInputError, widget.getInputValue)
        check_list = ('input', 'name="field.foo.add"')
        self.verifyResult(widget(), check_list)

    def test_add(self):
        request = TestRequest(form={'field.foo.add': u'Add bar',
                                    'field.foo.count': u'0'})
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        self.assert_(widget.hasInput())
        self.assertRaises(WidgetInputError, widget.getInputValue)
        check_list = (
            'checkbox', 'field.foo.remove_0', 'input', 'field.foo.0.bar',
            'submit', 'submit', 'field.foo.add'
        )
        self.verifyResult(widget(), check_list, inorder=True)

    def test_request(self):
        request = TestRequest(form={'field.foo.0.bar': u'Hello world!',
                                    'field.foo.count': u'1'})
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        self.assert_(widget.hasInput())
        self.assertEqual(widget.getInputValue(), (u'Hello world!',))

    def test_existing(self):
        request = TestRequest()
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        widget.setRenderedValue((u'existing',))
        self.assertFalse(widget.hasInput())
        self.assertRaises(MissingInputError, widget.getInputValue)
        check_list = (
            'checkbox', 'field.foo.remove_0', 'input', 'field.foo.0.bar',
            'existing', 'submit', 'submit', 'field.foo.add',
            'field.foo.count" value="1"',
        )
        self.verifyResult(widget(), check_list, inorder=True)
        widget.setRenderedValue((u'existing', u'second'))
        self.assertFalse(widget.hasInput())
        self.assertRaises(MissingInputError, widget.getInputValue)
        check_list = (
            'checkbox', 'field.foo.remove_0', 'input', 'field.foo.0.bar',
            'existing', 'checkbox', 'field.foo.remove_1', 'input', 'field.foo.1.bar',
            'second', 'submit', 'submit', 'field.foo.add', 'field.foo.count" value="2"',
        )
        self.verifyResult(widget(), check_list, inorder=True)

    def test_remove(self):
        request = TestRequest(form={
            'field.foo.remove_0': u'1',
            'field.foo.0.bar': u'existing', 'field.foo.1.bar': u'second',
            'field.foo.remove': u'Remove selected items',
            'field.foo.count': u'2'})
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        widget.setRenderedValue((u'existing', u'second'))
        self.assertEqual(widget.getInputValue(), (u'second',))
        check_list = (
            'checkbox', 'field.foo.remove_0', 'input', 'field.foo.0.bar',
            'existing', 'checkbox', 'field.foo.remove_1', 'input', 'field.foo.1.bar',
            'second', 'submit', 'submit', 'field.foo.add',
            'field.foo.count" value="2"',
        )
        self.verifyResult(widget(), check_list, inorder=True)

    def test_min(self):
        request = TestRequest()
        self.field.min_length = 2
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        widget.setRenderedValue((u'existing',))
        self.assertRaises(MissingInputError, widget.getInputValue)
        check_list = (
            'input', 'field.foo.0.bar', 'existing',
            'input', 'field.foo.1.bar', 'value=""',
            'submit', 'field.foo.add'
        )
        s = widget()
        self.verifyResult(s, check_list, inorder=True)
        self.assertEqual(s.find('checkbox'), -1)

    def test_max(self):
        request = TestRequest()
        self.field.max_length = 1
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        widget.setRenderedValue((u'existing',))
        self.assertRaises(MissingInputError, widget.getInputValue)
        s = widget()
        self.assertEqual(s.find('field.foo.add'), -1)

    def test_anonymousfield(self):
        self.field = Tuple(__name__=u'foo', value_type=TextLine())
        request = TestRequest()
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        widget.setRenderedValue((u'existing',))
        s = widget()
        check_list = (
            'input', '"field.foo.0."', 'existing',
            'submit', 'submit', 'field.foo.add'
        )
        s = widget()
        self.verifyResult(s, check_list, inorder=True)

    def test_usererror(self):
        self.field = Tuple(__name__=u'foo',
                           value_type=TextLine(__name__='bar'))
        request = TestRequest(form={
            'field.foo.0.bar': u'', 'field.foo.1.bar': u'nonempty',
            'field.foo.count': u'2'})
        widget = TupleSequenceWidget(
            self.field, self.field.value_type, request)
        # Rendering a widget should not raise errors!
        widget()

        data = widget._generateSequence()
        self.assertEqual(data, [None, u'nonempty'])

    def doctest_widgeterrors(self):
        """Test that errors on subwidgets appear

            >>> field = Tuple(__name__=u'foo',
            ...               value_type=TextLine(__name__='bar'))
            >>> request = TestRequest(form={
            ...     'field.foo.0.bar': u'',
            ...     'field.foo.1.bar': u'nonempty',
            ...     'field.foo.count': u'2'})
            >>> widget = TupleSequenceWidget(field, field.value_type, request)

         If we render the widget, we see no errors:

            >>> print widget()
            <BLANKLINE>
            ...
            <tr>
              <td>
                 <input class="editcheck" type="checkbox"
                        name="field.foo.remove_0" />
              </td>
              <td>
                 <input class="textType" id="field.foo.0.bar"
                        name="field.foo.0.bar"
                        size="20" type="text" value=""  />
              </td>
            </tr>
            ...

         However, if we call getInputValue or hasValidInput, the
         errors on the widgets are preserved and displayed:

            >>> widget.hasValidInput()
            False

            >>> print widget()
            <BLANKLINE>
            ...
            <tr>
              <td>
                 <input class="editcheck" type="checkbox"
                        name="field.foo.remove_0" />
              </td>
              <td>
                 <span class="error">Required input is missing.</span>
                 <input class="textType" id="field.foo.0.bar"
                        name="field.foo.0.bar"
                        size="20" type="text" value=""  />
              </td>
            </tr>
            ...
        """

    def test_presensemarker(self):
        request = TestRequest()
        widget = SequenceWidget(self.field, self.field.value_type, request)
        self.assertEqual(
            widget._getPresenceMarker(2),
            '<input type="hidden" '
            'name="%s.count" value="2" originalValue="2" />' % widget.name
        )

        # now pass object w/o foo attribute and check if it won't break
        class ITestContent(Interface):
            foo = self._FieldFactory(
                title=u'Foo',
                description=u'Foo Field',
            )

        class TestObject(object):
            implements(ITestContent)

        field = ITestContent['foo'].bind(TestObject())
        widget = SequenceWidget(field, field.value_type, request)
        self.assertEqual(
            widget._getPresenceMarker(1),
            '<input type="hidden" '
            'name="%s.count" value="1" />' % widget.name
        )



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Products.Five import BrowserView
from Products.Five.browser.decode import processInputs
from Products.Five.browser.decode import setPageEncoding
from zope.formlib import form as formlib

import json


class InlineValidationView(BrowserView):
    """Validate a form and return the error message for a particular field as JSON.
    """

    def __call__(self, fname):
        res = {'errmsg': ''}

        form = aq_inner(self.context)
        context = aq_inner(form.context)
        request = self.request
        processInputs(request)
        setPageEncoding(request)

        raw_fname = fname[len(form.prefix) + 1:]
        formlib_field = form.form_fields[raw_fname]

        widgets = formlib.setUpWidgets(
            (formlib_field,), form.prefix, context,
            request, form=form, adapters={}, ignore_request=False)
        widget = widgets[raw_fname]
        error = None
        try:
            widget.getInputValue()
        except:
            pass
        error = widget.error()

        res['errmsg'] = error or ''
        self.request.response.setHeader('Content-Type', 'application/json')
        return json.dumps(res)



# -*- coding: utf-8 -*-
from zope.component import testing
from zope.testing import cleanup
from Zope2.App import zcml

import doctest
import unittest


def setUp(test):
    import five.formlib
    import plone.app.form
    import plone.memoize
    import Products.CMFCore
    import Products.Five

    zcml.load_config('configure.zcml', Products.Five)
    zcml.load_config('configure.zcml', five.formlib)
    zcml.load_config('permissions.zcml', Products.CMFCore)
    zcml.load_config('configure.zcml', plone.app.form)
    zcml.load_config('configure.zcml', plone.memoize)


def tearDown(test):
    cleanup.cleanUp()


def test_suite():
    optionflags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)
    return unittest.TestSuite((
        doctest.DocFileSuite(
            'formlib.txt',
            package='plone.app.form',
            setUp=testing.setUp,
            tearDown=testing.tearDown),
        doctest.DocFileSuite(
            'inline_validation.txt',
            package='plone.app.form',
            setUp=setUp,
            tearDown=tearDown,
            optionflags=optionflags),
    ))



# -*- coding: utf-8 -*-
from checkboxwidget import CheckBoxWidget  # noqa
from checkboxwidget import DisabledCheckBoxWidget  # noqa
from multicheckboxwidget import MultiCheckBoxWidget  # noqa
from selectionwidget import LanguageDropdownChoiceWidget  # noqa
from sequencewidget import ListSequenceWidget  # noqa
from sequencewidget import SequenceWidget  # noqa
from sequencewidget import TupleSequenceWidget  # noqa



# -*- coding: utf-8 -*-
"""Event definitions
"""
from plone.app.form.interfaces import IEditBegunEvent
from plone.app.form.interfaces import IEditCancelledEvent
from plone.app.form.interfaces import IEditSavedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class EditBegunEvent(ObjectEvent):
    """An edit operation was begun
    """
    implements(IEditBegunEvent)


class EditCancelledEvent(ObjectEvent):
    """An edit operation was cancelled
    """
    implements(IEditCancelledEvent)


class EditSavedEvent(ObjectEvent):
    """An edit operation was completed
    """
    implements(IEditSavedEvent)



# -*- coding: utf-8 -*-
from plone.app.form.testing import INTEGRATION_TESTING
from plone.app.form.widgets.wysiwygwidget import WYSIWYGWidget

import unittest


BODY = """<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en"
      xmlns:tal="http://xml.zope.org/namespaces/tal"
      xmlns:metal="http://xml.zope.org/namespaces/metal"
      xmlns:i18n="http://xml.zope.org/namespaces/i18n"
      i18n:domain="plone">

<div metal:define-macro="wysiwygEditorBox">
  <div>Cool Editor Box</div>
  <textarea name="description"
            rows="25"
            tal:content="inputvalue"
            tal:attributes="name inputname;
                            id inputname;"></textarea>
</div>

</html>
"""


class WYSIWYGWidgetTestCase(unittest.TestCase):
    """Base class used for test cases
    """

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.pm = self.portal.portal_membership
        self.skins = self.portal.portal_skins

    def test_right_macro(self):
        # fixes #8016
        class MyField:
            __name__ = 'the field'
            required = True
            default = u'the value'
            missing_value = None
            title = ""
            description = ""

        # Let's add a custom editor using our own widget html:
        from zope.pagetemplate.pagetemplatefile import PageTemplate
        template = PageTemplate()
        template.pt_edit(BODY, 'text/html')

        self.skins.custom.cool_editor_wysiwyg_support = template

        # The wysiwyg widget depends on the used editor.
        # Let's change it to `cool_editor`.
        pm = self.pm
        member = pm.getAuthenticatedMember()
        member.setMemberProperties({'wysiwyg_editor': 'cool_editor'})

        w = WYSIWYGWidget(MyField(), self.request)
        # The test is partially that this call does not give an error:
        html = w()
        # This is true for standard Plone as well:
        self.assertIn(
            '<textarea name="field.the field" rows="25" id="field.the field">'
            'the value</textarea>', html)
        # Only our cool editor has this:
        self.assertIn('Cool Editor Box', html)



# -*- coding: utf-8 -*-
from zope.formlib.itemswidgets import DropdownWidget
from zope.component import queryMultiAdapter


class LanguageDropdownChoiceWidget(DropdownWidget):
    """ A DropdownWidget which renders a localized language selection.
    """

    def __init__(self, field, request):
        """Initialize the widget."""
        super(LanguageDropdownChoiceWidget, self).__init__(
            field,
            field.vocabulary,
            request
        )

    def renderItemsWithValues(self, values):
        """Render the list of possible values, with those found in
        `values` being marked as selected."""

        # Sort languages by their title
        portal_state = queryMultiAdapter((self.context, self.request),
                                         name=u'plone_portal_state')
        languages = portal_state.locale().displayNames.languages

        terms = []
        languages = languages
        for term in self.vocabulary:
            value = term.value
            title = languages.get(value, term.title)
            if title == value:
                title = term.title
            terms.append((title, term.token))

        terms.sort()

        cssClass = self.cssClass
        name = self.name
        renderSelectedItem = self.renderSelectedItem
        renderItem = self.renderItem
        rendered_items = []
        count = 0

        for title, token in terms:
            if token in values:
                render = renderSelectedItem
            else:
                render = renderItem

            rendered_item = render(count, title, token, name, cssClass)
            rendered_items.append(rendered_item)
            count += 1

        return rendered_items



# -*- coding: utf-8 -*-
from zope.formlib.boolwidgets import CheckBoxWidget as BaseWidget
from zope.formlib.widget import renderElement
from zope.i18n import translate
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('plone')


class CheckBoxWidget(BaseWidget):
    """ Plone specific widget that is going to show the checkbox widget on the left of the label
        in order to do that we remove the title / label / required
    """

    def __init__(self, context, request):
        BaseWidget.__init__(self, context, request)
        self.required = False
        self.__required = context.required
        self.label = ""
        self.hint = ""

    disabled = False

    def __call__(self):
        """Render the widget to HTML."""
        value = self._getFormValue()
        html = u"<label for='{0}'>{1}".format(
            self.name,
            translate(self.context.title, context=self.request)
        )
        if self.__required:
            # Use the numeric character reference here instead of &nbsp; to make
            # our xml-parsing tests happier.
            html += u" <span class='required' title='{0}'>&#160;</span>".format(
                translate(_(u'title_required', default='Required'), context=self.request)
            )
        if self.context.description:
            html += u" <span class='formHelp'>{0}</span>".format(
                translate(self.context.description, context=self.request)
            )
        html += u"</label>\n"

        if value == 'on':
            kw = {'checked': 'checked'}
        else:
            kw = {}
        if self.disabled:
            kw['disabled'] = 'disabled'
        return u'{0}  {1} {2}'.format(
            renderElement(self.tag,
                          type='hidden',
                          name=self.name + ".used",
                          id=self.name + ".used",
                          value=""
                          ),
            renderElement(self.tag,
                          type=self.type,
                          name=self.name,
                          id=self.name,
                          cssClass=self.cssClass,
                          extra=self.extra,
                          value="on",
                          **kw),
            html
        )


class DisabledCheckBoxWidget(CheckBoxWidget):
    """Simple variation of the CheckBoxWidget which renders itself disabled.
    """

    disabled = True



# -*- coding: utf-8 -*-
from zope.testing import cleanup
from Zope2.App import zcml

import doctest
import unittest


def setUp(test):
    import Products.Five
    import Products.CMFCore
    import plone.app.form

    zcml.load_config('configure.zcml', Products.Five)
    zcml.load_config('permissions.zcml', Products.CMFCore)
    zcml.load_config('configure.zcml', plone.app.form)


def tearDown(test):
    cleanup.cleanUp()


def test_suite():
    optionflags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)
    return unittest.TestSuite([
        doctest.DocFileSuite('widgets/uberselectionwidget.txt',
                             package='plone.app.form',
                             setUp=setUp,
                             tearDown=tearDown,
                             optionflags=optionflags),
        doctest.DocFileSuite('widgets/checkboxwidget.txt',
                             package='plone.app.form',
                             setUp=setUp,
                             tearDown=tearDown,
                             optionflags=optionflags),
    ])



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import BoundPageTemplate
from zope import interface
from zope.formlib import namedtemplate

import new

try:
    # chameleon-compatible page templates
    from five.pt.pagetemplate import ViewPageTemplateFile as ChameleonPageTemplateFile
    HAS_CHAMELEON = True
except ImportError:
    HAS_CHAMELEON = False


class NamedTemplateAdapter(object):
    """A named template adapter implementation that has the ability
    to lookup the template portion from regular traversal (intended for
    being able to customize the template portion of a view component
    in the traditional portal_skins style).
    """

    interface.implements(namedtemplate.INamedTemplate)

    def __init__(self, context):
        self.context = context

    @property
    def macros(self):
        return self.default_template.macros

    def __call__(self, *args, **kwargs):
        context = aq_inner(self.context)
        context_of_context = aq_inner(context.context)
        view = context.__of__(context_of_context)

        # self.default_template is a ViewPageTemplateFile, which is a property descriptor
        # whose __get__ method returns a BoundPageTemplate.  That expects to be accessed from
        # a view, but we're accessing it from a NamedTemplateAdapter so we have to be sneaky and
        # make our own BoundPageTemplate rather than calling self.default_template directly.
        return BoundPageTemplate(self.__class__.__dict__['default_template'], view)(*args, **kwargs)


def named_template_adapter(template):
    """Return a new named template adapter which defaults the to given
    template.
    """

    new_class = new.classobj('GeneratedClass',
                             (NamedTemplateAdapter,),
                             {})
    if HAS_CHAMELEON:
        template = ChameleonPageTemplateFile(template.filename)
    new_class.default_template = template
    return new_class



# -*- coding: utf-8 -*-
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.browser.interfaces import ITerms
from zope.component import getMultiAdapter
from zope.formlib.interfaces import ISourceQueryView
from zope.formlib.interfaces import IWidgetInputErrorView
from zope.formlib.interfaces import WidgetInputError
from zope.formlib.widget import SimpleInputWidget
from zope.schema.interfaces import ValidationError


class UberSelectionWidget(SimpleInputWidget):
    _error = None

    template = ViewPageTemplateFile('uberselectionwidget.pt')
    limit_results = 20

    def __init__(self, field, request):
        SimpleInputWidget.__init__(self, field, request)
        self.source = field.source
        self.terms = getMultiAdapter((self.source, self.request), ITerms)
        self.queryview = getMultiAdapter((self.source, self.request), ISourceQueryView)

    def _value(self):
        if self._renderedValueSet():
            value = self._data
        else:
            token = self.request.form.get(self.name)

            if token is not None and token != '':
                if not isinstance(token, basestring):  # noqa
                    token = token[-1]
                try:
                    value = self.terms.getValue(str(token))
                except LookupError:
                    value = self.context.missing_value
            else:
                value = self.context.missing_value

        return value

    def _getRenderValue(self):
        value = self._value()
        if value is not None:
            value = self.terms.getTerm(value)
        return value

    def hidden(self):
        value = self._value()
        if value == self.context.missing_value:
            return ''  # Nothing to hide ;)

        try:
            term = self.terms.getTerm(value)
        except LookupError:
            # A value was set, but it's not valid.  Treat
            # it as if it was missing and return nothing.
            return ''

        return '<input type="hidden" name="{0}" value="{1}" />'.format(
            self.name,
            term.token
        )

    def error(self):
        if self._error:
            return getMultiAdapter((self._error, self.request),
                                   IWidgetInputErrorView).snippet()
        return ""

    def __call__(self):
        value = self._getRenderValue()
        field = self.context
        results = []
        results_truncated = False
        limit = self.limit_results
        qresults = self.queryview.results(self.name)
        if qresults is not None:
            for index, item in enumerate(qresults):
                if (limit > 0 and index >= limit):
                    results_truncated = True
                    break
                results.append(self.terms.getTerm(item))
        return self.template(field=field,
                             results=results,
                             results_truncated=results_truncated,
                             name=self.name,
                             value=value)

    def getInputValue(self):
        value = self._value()

        field = self.context

        # Remaining code copied from SimpleInputWidget

        # value must be valid per the field constraints
        try:
            field.validate(value)
        except ValidationError as err:
            self._error = WidgetInputError(field.__name__, self.label, err)
            raise self._error

        return value

    def hasInput(self):
        if self.name in self.request or self.name+'.displayed' in self.request:
            return True

        token = self.request.form.get(self.name)
        if token is not None:
            return True

        return False


class UberMultiSelectionWidget(UberSelectionWidget):
    template = ViewPageTemplateFile('ubermultiselectionwidget.pt')

    def __init__(self, field, request):
        SimpleInputWidget.__init__(self, field, request)
        self.source = field.value_type.source
        self.terms = getMultiAdapter((self.source, self.request), ITerms)
        self.queryview = getMultiAdapter((self.source, self.request), ISourceQueryView)

    def _value(self):
        if self._renderedValueSet():
            value = self._data
        else:
            tokens = self.request.form.get(self.name)

            if tokens is not None:
                value = []
                for token in tokens:
                    try:
                        v = self.terms.getValue(str(token))
                    except LookupError:
                        pass  # skip invalid values
                    else:
                        value.append(v)
                # only keep unique items
                r = []
                seen = {}
                for s in value:
                    if s not in seen:
                        r.append(s)
                        seen[s] = 1
                value = r
            else:
                if self.name+'.displayed' in self.request:
                    value = []
                else:
                    value = self.context.missing_value
        if value is None:
            value = []
        return value

    def _getRenderValue(self):
        value = self._value()
        if value is not None:
            value = [self.terms.getTerm(x) for x in value]
        return value



# -*- coding: utf-8 -*-
from zope.formlib.sequencewidget import SequenceWidget as BaseWidget


class SequenceWidget(BaseWidget):
    """Plone specific widget that is going to include originalValue attribute
    within marker input element.

    It'll be used to enable unload form protection for Remove, Add widget
    buttons usage.
    """

    def _getPresenceMarker(self, count=0):
        # insert originalValue attribute if there is context
        orig = ''
        # Not all content objects must necessarily support the attributes
        if self.context.context is not None and hasattr(self.context.context,
           self.context.__name__):
            orig = ' originalValue="{0}"'.format(
                len(self.context.get(self.context.context))
            )
        return ('<input type="hidden" name="{0}.count" value="{1}"{2} />'.format(
            self.name,
            count,
            orig)
        )


class TupleSequenceWidget(SequenceWidget):
    _type = tuple


class ListSequenceWidget(SequenceWidget):
    _type = list



# -*- coding: utf-8 -*-
from zope.component.interfaces import IObjectEvent
from zope.formlib.interfaces import IPageForm
from zope.formlib.interfaces import ISubPageForm
from zope.interface import Attribute
from zope.interface import Interface


class IPlonePageForm(IPageForm):
    """A page form with a couple extra attributes
    """
    description = Attribute('A longer description to display on the form')
    form_name = Attribute('A label to apply to the fieldset')


class IPloneSubPageForm(ISubPageForm):
    """A page form with a couple extra attributes
    """
    description = Attribute('A longer description to display on the form')
    form_name = Attribute('A label to apply to the fieldset')


class IEditForm(Interface):
    """Marker interface for edit forms. This allows things like the locking
    widget to be registered for edit forms only.
    """


class IEditBegunEvent(IObjectEvent):
    """An event signalling that editing has begun on an object
    """


class IEditFinishedEvent(IObjectEvent):
    """Base event signalling that an edit operation has completed
    """


class IEditCancelledEvent(IEditFinishedEvent):
    """An event signalling that editing was cancelled on the given object
    """


class IEditSavedEvent(IEditFinishedEvent):
    """An event signalling that editing was complated on the given object
    """



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

version = '2.3.2'

long_description = (open("README.rst").read() + "\n" +
                    open("CHANGES.rst").read())

setup(name='plone.app.form',
      version=version,
      description="zope.formlib integration for Plone",
      long_description=long_description,
      classifiers=[
          "Environment :: Web Environment",
          "Framework :: Plone",
          "Framework :: Zope2",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
      ],
      keywords='plone form formlib',
      author='Plone Foundation',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://pypi.python.org/pypi/plone.app.form',
      license='GPL version 2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone', 'plone.app'],
      include_package_data=True,
      zip_safe=False,
      extras_require=dict(
          test=[
              'plone.app.content',
              'plone.app.testing',
              'plone.memoize',
              'zope.annotation',
              'zope.publisher',
              'zope.testing',
          ],
      ),
      install_requires=[
          'Acquisition',
          'DateTime',
          'five.formlib',
          'plone.app.vocabularies',
          'plone.locking',
          'Products.CMFCore',
          'Products.CMFDefault',
          'setuptools',
          'zope.browser',
          'zope.component',
          'zope.event',
          'zope.formlib >= 4.0',
          'zope.i18n',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.lifecycleevent',
          'zope.schema',
          'zope.site',
          'Zope2 >= 2.12.3',
      ],
      )



# -*- coding: utf-8 -*-
from five.formlib import formbase

import logging


def apply_patches():
    # Five's base class defines template as a ViewPageTemplateFile.
    # However, the base 'zope.formlib.form.FormBase' has 'template'
    # defined as a NamedTemplate, which is much nicer, and what we
    # want.
    if 'template' in formbase.FiveFormlibMixin.__dict__:
        logger = logging.getLogger('plone.app.form')
        logger.debug(
            '*** MONKEYPATCH *** : delete "template" attribute of '
            'FiveFormlibMixin to allow use of named templates.'
        )
        delattr(formbase.FiveFormlibMixin, 'template')
